package org.firstinspires.ftc.teamcode.subsystem;

import static org.firstinspires.ftc.teamcode.PedroConstants.ARM_SERVO;
import static org.firstinspires.ftc.teamcode.configs.RobotConstants.armBucket;
import static org.firstinspires.ftc.teamcode.configs.RobotConstants.armFloor;
import static org.firstinspires.ftc.teamcode.configs.RobotConstants.armGrabBlueberrySkyhook;
import static org.firstinspires.ftc.teamcode.configs.RobotConstants.armHangBlueberrySkyhook;
import static org.firstinspires.ftc.teamcode.configs.RobotConstants.armInit;
import static org.firstinspires.ftc.teamcode.configs.RobotConstants.armPark;
import static org.firstinspires.ftc.teamcode.configs.RobotConstants.armReverseBucket;

import com.qualcomm.robotcore.hardware.HardwareMap;
import com.qualcomm.robotcore.hardware.ServoImplEx;

public class ArmSubsystem {

    public enum ArmState {
        PARK, FLOOR, BUCKET, SUBMARINE, GRAB_BLUEBERRY_SKYHOOK, HANG_BLUEBERRY_SKYHOOK, INIT, REVERE_BUCKET
    }

    private ServoImplEx arm;
    private ArmState state;

    public ArmSubsystem(HardwareMap hardwareMap) {
        this.arm = hardwareMap.get(ServoImplEx.class, ARM_SERVO);
    }
    public void grabBlueberrySkyhook(){
        setState(ArmState.GRAB_BLUEBERRY_SKYHOOK);
        arm.setPosition(armGrabBlueberrySkyhook);
    }
    public void hangBlueberrySkyhook(){
        setState(ArmState.HANG_BLUEBERRY_SKYHOOK);
        arm.setPosition(armHangBlueberrySkyhook);
    }
    public void toReverseBucket(){
        arm.setPosition(armReverseBucket);
        setState(ArmState.REVERE_BUCKET);
    }
    public void toParkPosition() {
        arm.setPosition(armPark);
        setState(ArmState.PARK);
    }

    public void toFloorPosition() {
        arm.setPosition(armFloor);
        setState(ArmState.FLOOR);
    }

    public void toBucketPosition() {
        arm.setPosition(armBucket);
        setState(ArmState.BUCKET);
    }
    public void toInitPosition(){
        arm.setPosition(armInit);
        setState(ArmState.INIT);
    }

    public void setState(ArmState armState) {
        this.state = armState;
    }

    public ArmState getState() {
        return this.state;
    }

    public void initTeleOp() {
        arm.resetDeviceConfigurationForOpMode();
        toParkPosition();
    }
    public void initAuto(){
        arm.resetDeviceConfigurationForOpMode();
        toInitPosition();
    }

    public double getPosition() {
        return this.arm.getPosition();
    }

    public void setPosition(double position) {
        this.arm.setPosition(position);
    }

}
